function D = D_SP(P, N, lb, ub, delta)
% D = D_SP(P, N, lb, ub, delta)
%
% Computes the Solow-Polasky measure
%
% Authors: Johannes W. Kruisselbrink, Michael T.M. Emmerich, Andre H. Deutz

	if (nargin < 5)
		% Set delta appropriately
		delta = 1;
	end

	% If P is empty, then D = inf
	if (isempty(P))
		D = 0;
		return;
	end

	% Compute distance matrix
	dmat = zeros(size(P,2),size(P,2));
	for i = 1:size(P,2)
		for j = 1:size(P,2)
			dmat(i,j) = norm(P(:,i) - P(:,j));
		end
	end

	M = exp(-delta .* dmat);

	C = pinv(M);

	D = - sum(sum(C));

end
