function Xs = grid_sampling(m, N, lb, ub)
% grid_sampling(m, N, lb, ub)
%
% Construct sample points of an ND-grid
%
% Input:
% - lb          - An 1xN vector of lower bounds of data
% - ub          - An 1xN vector of upper bounds of data
% - N           - The number of dimensions
% - m           - The number of desired grid points
%                 (should be a power of 2)
%
% Output:
% - Xs          - The mxN sample points
%
% Authors: Johannes W. Kruisselbrink, Michael T.M. Emmerich, Andre H. Deutz

	% The resolution in every dimension
	resolution = max(2, ceil(nthroot(m, N)));
	m = resolution^N;

	% Initialize sample set
	Xs = zeros(m,N);

	% Compute the grid coordinates for each individual dimension
	for i = 1:N
		gi(:,i) = linspace(lb(i), ub(i), resolution);
	end

	rep = 1;
	for i = 1:N
		r_count = 1;
		r_ind = 1;
		for j = 1:m
			Xs(j,i) = gi(r_ind, i);
			r_count = r_count + 1;
			if (r_count > rep)
				r_ind = mod(r_ind, resolution) + 1;
				r_count = 1;
			end
		end
		rep = rep * resolution;
	end
end
